﻿/*******************************************************************************
* Copyright (C) 2012-2013 Intel Corp. All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of source code must retain the above copyright notice,
*    this list of conditions and the following disclaimer.
*
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this list of conditions and the following disclaimer in the documentation
*    and/or other materials provided with the distribution.
*
*  - Neither the name of Intel Corp. nor the names of its
*    contributors may be used to endorse or promote products derived from this
*    software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL Intel Corp. OR THE CONTRIBUTORS
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Diagnostics;

namespace TBT_FW_Update_Tool
{
    public enum FwUpdateExecStatus {
        UpdateSuccess = 100,
        FileUnpackFailure,
        DllLoadFailure,
        NoDriverLoaded,
        ImageWriteFailure,
        AnotherError
    }

    class FwUpdateModule
    {
        public FwUpdateModule()
        { 
       
        }

        public FwUpdateExecStatus ExecuteUpdateFlow(byte[] image)
        {

            FwUpdateExecStatus tempStatus = CheckDriverConnection();
            if (tempStatus != FwUpdateExecStatus.UpdateSuccess)
            {
                return tempStatus;
            }
            else
            {

                FwUpdateExecStatus imageWriteStatus = FullImageWriteFlow(image);
                if (imageWriteStatus != FwUpdateExecStatus.UpdateSuccess)
                {
                    return imageWriteStatus;
                }
            }

            return FwUpdateExecStatus.UpdateSuccess;
        }

        internal FwUpdateExecStatus ShowFwVersion()
        {
           
            MessageBox.Show("Current NVM Version: " + getFWVersion().ToString("X") + "\n");

            return FwUpdateExecStatus.UpdateSuccess;
        }

        internal uint getFWVersion()
        {
            uint libCoreStatus;
            uint nvm_version=0;

          
            libCoreStatus = FwUpdateDllApi.FwUpdateGetCurrentNvmVersion(ref nvm_version);

            if (libCoreStatus != 0)
            {
                 throw new Exception("error getting the FW version");
            }
           

            return nvm_version;
        }

        private FwUpdateExecStatus FullImageWriteFlow(byte[] image)
        {
            uint libCoreStatus;

            libCoreStatus = FwUpdateDllApi.FwUpdatePerformUpdate((uint)image.Length, image);

            if (libCoreStatus != 0)
            {
                return FwUpdateExecStatus.ImageWriteFailure;
            }
           
            return FwUpdateExecStatus.UpdateSuccess;
        }


        private FwUpdateExecStatus CheckDriverConnection()
        {        
            int triesLeft = 3;
            UInt32 status;
            while( triesLeft > 0)
            {
                status = FwUpdateDllApi.FwUpdateCheckDriverConnection();
                --triesLeft;

                if (status == 0) //driver is loaded
                {
                    return FwUpdateExecStatus.UpdateSuccess;
                }
                else //driver is not loaded
                {
                    if (triesLeft > 0)
                    {
                        DialogResult result = MessageBox.Show("There is no Thunderbolt\u2122 device connected to the system\n" +
                                                               "or Thunderbolt\u2122 Software is not present in the system.\n" +
                                                                "Make sure you have Thunderbolt\u2122 Software installed.\n" +
                                                                "Then please connect Thunderbolt\u2122 device and press Retry or press Cancel to abort Firmware update process",
                                                                "Thunderbolt\u2122 Firmware update utility",
                                                                MessageBoxButtons.RetryCancel,
                                                                MessageBoxIcon.Warning);

                        if (result == DialogResult.Cancel)
                        {
                            return FwUpdateExecStatus.NoDriverLoaded;
                        }
                    }
                   
                }

            }
            MessageBox.Show("The update process cannot be performed because there is no device connected to Thunderbolt\u2122 port or Thunderbolt\u2122 Software is not present in the system.\n Aborting...", "Thunderbolt\u2122 Firmware update utility", MessageBoxButtons.OK, MessageBoxIcon.Error);
            return FwUpdateExecStatus.NoDriverLoaded;

        }

    }
}
