﻿/*******************************************************************************
* Copyright (C) 2012-2013 Intel Corp. All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of source code must retain the above copyright notice,
*    this list of conditions and the following disclaimer.
*
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this list of conditions and the following disclaimer in the documentation
*    and/or other materials provided with the distribution.
*
*  - Neither the name of Intel Corp. nor the names of its
*    contributors may be used to endorse or promote products derived from this
*    software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL Intel Corp. OR THE CONTRIBUTORS
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using TBT_FW_Update_Tool.Properties;
using System.IO;
using System.Diagnostics;


namespace TBT_FW_Update_Tool
{
    public enum programFlowState { WelcomeScreen, FileChooseScreen, StartingScreen, ProgressScreen}

    public partial class MainWindow : Form
    {
    
        private String FWfile;
        private Thread m_UpdateThread;
        private programFlowState m_State;

        public MainWindow()
        {
            InitializeComponent();
            UnpackFiles();
            m_State = programFlowState.WelcomeScreen;
            ShowWelcomeScreen();
            this.Text = "Thunderbolt\u2122 Firmware Update utility";
        }

        private void ShowWelcomeScreen()
        {
            panelWelcome.Visible = true;
            panelStartProcess.Visible = false;
            panelProgress.Visible = false;
            buttonBack.Enabled = false;
            buttonContinue.Enabled = true;
            buttonContinue.Visible = true;
            buttonCancel.Enabled = true;
            m_State = programFlowState.WelcomeScreen;
          
        }

        private void ShowFileChooseScreen()
        {
            panelWelcome.Visible = false;
            panelStartProcess.Visible = false;
            panelProgress.Visible = false;
            panelChooseFile.Visible = true;
            buttonBack.Enabled = true;
            buttonContinue.Enabled = true;
            buttonContinue.Visible = true;
            buttonCancel.Enabled = true;
            m_State = programFlowState.FileChooseScreen;
        }

        private void ShowStartingScreen()
        {
            panelWelcome.Visible = false;
            panelChooseFile.Visible = false;
            panelStartProcess.Visible = true;
            panelProgress.Visible = false;          
            buttonBack.Enabled = true;
            buttonContinue.Enabled = false;
            buttonContinue.Visible = false;
            buttonCancel.Enabled = true;
            FWVersion.Text = new FwUpdateModule().getFWVersion().ToString("X");
            m_State = programFlowState.StartingScreen;
        }

        private void ShowProgressScreen()
        {
            panelWelcome.Visible = false;
            panelStartProcess.Visible = false;
            panelProgress.Visible = true;         
            buttonBack.Enabled = false;
            buttonContinue.Enabled = false;
            buttonContinue.Visible = false;
            buttonCancel.Enabled = false;
            m_State = programFlowState.ProgressScreen;
        }

        private void buttonContinue_Click(object sender, EventArgs e)
        {
            if (m_State == programFlowState.WelcomeScreen)
            {
                ShowFileChooseScreen();
            }
            else if (m_State == programFlowState.FileChooseScreen)
            {
                FWfile = textBox1.Text;
                ShowStartingScreen();
            }
        }

        // This function is called after click on "Start FW Update" button
        private void UpdateProcessEntry(bool certified)
        {
            ShowProgressScreen();
            FwUpdateExecStatus status = DoPreparations();       //Unpacking and loading library
            if (status != FwUpdateExecStatus.UpdateSuccess)
            {
                FinalGuiUpdate(status);
                return;
            }

            //FW Update is called in separated thread
            m_UpdateThread = new Thread(new ThreadStart(DoUpdate));
            m_UpdateThread.Start();
        }

        private FwUpdateExecStatus DoPreparations()
        {
            try
            {
           
                buttonCancel.Enabled = false;
                buttonContinue.Enabled = false;


              
                

                return FwUpdateExecStatus.UpdateSuccess;
            }
            catch (Exception)
            {
                return FwUpdateExecStatus.AnotherError;
            }
            
        }

        private void DoUpdate()
        {
            FwUpdateExecStatus status = ExecuteUpdateFlow();
            FinalGuiUpdate(status);
        }

        private void FinalGuiUpdate(FwUpdateExecStatus status)
        {
            this.Invoke((MethodInvoker)delegate()
            {
                buttonCancel.Enabled = true;
                buttonCancel.Text = "Finish";
                
                if (status == FwUpdateExecStatus.UpdateSuccess)
                {
                    labelProgressMessage.Text = "Firmware was updated successfully";             
                }
                else if (status == FwUpdateExecStatus.NoDriverLoaded)
                {
                    labelProgressMessage.Text = "Firmware update was aborted due to no Thunderbolt\u2122 devices connected to the system";
                }
                else
                {
                    labelProgressMessage.Text = "Firmware update failed. Make sure that at least one Thunderbolt\u2122 device is connected and retry.\nIf the problem persists - contact your PC vendor for support.";  
                }        
                Update();
            });
        }

       
        private FwUpdateExecStatus ExecuteUpdateFlow()
        {
            try
            {
                FwUpdateExecStatus status = FwUpdateExecStatus.UpdateSuccess;

                this.Invoke((MethodInvoker)delegate() {
                    Update();
                });

                FwUpdateModule fuModule = new FwUpdateModule();

                //loading binary FW image
                try
                {
                   FileStream fs = new FileStream(FWfile, FileMode.Open, FileAccess.Read);
                   int size = (int)fs.Length;
                   BinaryReader reader = new BinaryReader(fs);
                   byte[] imageBuffer = new byte[size];
                   imageBuffer = reader.ReadBytes(size);
                   status = fuModule.ExecuteUpdateFlow(imageBuffer);
                }
                catch (Exception)
                {
                   return FwUpdateExecStatus.AnotherError;
                }

                if (status == FwUpdateExecStatus.NoDriverLoaded)
                {
                    labelProgressMessage.Text = "Firmware update is canceling...";
                }
                return status;
            }
            catch (Exception)
            {
                return FwUpdateExecStatus.AnotherError;
            }
        }

        private void UnpackFiles()
        {
            try
            {
#if _X86
                SaveResourceToDisc(Resources_x86.FwUpdateSdkWin32, Environment.GetFolderPath(Environment.SpecialFolder.System) + "\\FwUpdateSdk.dll");
#endif
#if _X64              
                SaveResourceToDisc(Resources_x64.FwUpdateSdkx64, Environment.GetFolderPath(Environment.SpecialFolder.System) + "\\FwUpdateSdk.dll");
#endif
            }
            catch (Exception e)
            {
                throw new Exception("Error unpacking files", e);
            }
        }

        private void SaveResourceToDisc(byte[] resource, string recourceFile)
        {
            File.Delete(recourceFile);
            Stream output = new FileStream(recourceFile, FileMode.CreateNew, FileAccess.Write, FileShare.Write);
            output.Write(resource, 0, resource.Length);
            output.Close();
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void MainWindow_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (!buttonCancel.Enabled)
            {
                if (e.CloseReason == CloseReason.TaskManagerClosing || e.CloseReason == CloseReason.UserClosing)
                {
                    e.Cancel = true;
                    MessageBox.Show("Please wait until the Thunderbolt\u2122 Firmware Update finishes it's activity and exit it using Finish button. Terminating it in other ways may leave your system in an unpredicted state.", "Thunderbolt\u2122 Firmware Update", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    return;
                }
            }
        }

        private void buttonBack_Click(object sender, EventArgs e)
        {
            if (m_State == programFlowState.FileChooseScreen)
            {
                ShowWelcomeScreen();
            }
            else if (m_State == programFlowState.StartingScreen)
            {
                ShowFileChooseScreen();
            }
            
        }

        private void panelStartFwUpdateProcess_MouseLeave(object sender, EventArgs e)
        {
            panelStartFwUpdateProcess.BackColor = Color.White;
            labelCertQuestioWinOnlyMain.BackColor = Color.White;
            labelCertQuestionWinOnlySmall.BackColor = Color.White;
            labelCertQuestioWinOnlyMain.ForeColor = Color.MidnightBlue;
            labelCertQuestionWinOnlySmall.ForeColor = Color.MidnightBlue;
        }

        private void panelStartFwUpdateProcess_MouseEnter(object sender, EventArgs e)
        {
            panelStartFwUpdateProcess.BackColor = Color.Snow;
            labelCertQuestioWinOnlyMain.BackColor = Color.Snow;
            labelCertQuestionWinOnlySmall.BackColor = Color.Snow;
            labelCertQuestioWinOnlyMain.ForeColor = Color.RoyalBlue;
            labelCertQuestionWinOnlySmall.ForeColor = Color.RoyalBlue;                 
        }

        private void panelStartFwUpdateProcess_MouseDown(object sender, MouseEventArgs e)
        {
            panelStartFwUpdateProcess.BorderStyle = BorderStyle.FixedSingle;
        }

        private void panelStartFwUpdateProcess_MouseUp(object sender, MouseEventArgs e)
        {
            panelStartFwUpdateProcess.BorderStyle = BorderStyle.None;
        }

        private void panelStartFwUpdateProcess_MouseClick(object sender, MouseEventArgs e)
        {
            UpdateProcessEntry(true);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            DialogResult result = openFileDialog.ShowDialog();
            if (result == DialogResult.OK)
            {
                textBox1.Text = openFileDialog.FileName;
            }
        }

    }
}
